`timescale 1ns/1ps
module uart_tb;

  // Parameters
  parameter CLK_PERIOD = 10;
  parameter DATA_WIDTH = 8;
  parameter FREQ = 100000000;
  parameter BAUD = 115200;


  //Ports
  reg  i_clk;
  reg  i_rst;
  wire  o_tx;
  wire  i_rx;

  // DUT SIGNALI
  wire [DATA_WIDTH-1:0] i_tx_data_loopBack;
  wire  i_tx_valid_loopBack;
  wire  o_tx_busy_loopBack;
  wire [DATA_WIDTH-1:0] o_rx_data_loopBack;
  wire  o_rx_valid_loopBack;

  // TB DUT SIGNALI
  reg [DATA_WIDTH-1:0] i_tx_data;
  reg  i_tx_valid;
  wire  o_tx_busy;
  wire [DATA_WIDTH-1:0] o_rx_data;
  wire  o_rx_valid;
  
  reg i_button;


top  #(.DATA_WIDTH(DATA_WIDTH), .FREQ(FREQ), .BAUD(BAUD)) top_inst(
    .i_clk(i_clk),
    .i_rst(i_rst),

    .o_tx(o_tx),
    .i_rx(i_rx),
    
    .i_button(i_button)
);

  uart #(.DATA_WIDTH(DATA_WIDTH), .FREQ(FREQ), .BAUD(BAUD))  
  DUT_TB (
    .i_clk(i_clk),
    .i_rst(i_rst),
    .i_tx_data(i_tx_data),
    .i_tx_valid(i_tx_valid),
    .o_tx_busy(o_tx_busy),
    .o_rx_data(o_rx_data),
    .o_rx_valid(o_rx_valid),
    .o_tx(i_rx),
    .i_rx(o_tx)
  );

  always #(CLK_PERIOD/2)  i_clk = !i_clk ;

  initial begin
    $timeformat(-9, 2, " ns");
    i_clk = 0;
    i_rst = 1;
    i_tx_data = 0;
    i_tx_valid = 0;
    i_tx_data = 0;
    i_tx_valid = 0;
    i_button = 0;
    #(CLK_PERIOD*10);
    i_rst = 0;
    #(CLK_PERIOD*10);
    i_button = 1;
    #(CLK_PERIOD*10);
    i_button = 0;
    #(CLK_PERIOD*10);    
    write_data_uart(8'h04);
    wait_clk_tick(1);
    write_data_uart(8'h05);
    wait_clk_tick(1);
    write_data_uart(8'h00);
end

integer num_tick_cpu;
task wait_clk_tick(input [15:0] num_clk_tick);
  begin
    for (num_tick_cpu = 0; num_tick_cpu < num_clk_tick; num_tick_cpu = num_tick_cpu + 1)
    begin
      @(posedge i_clk);
    end
  end
endtask


always @(posedge o_rx_valid) begin
    $display("@%0t RX data  : %0h", $realtime, o_rx_data);
end

task write_data_uart(input [7:0] data);
begin
  i_tx_valid <= 1'b1;
  i_tx_data <= data;
  @(posedge o_tx_busy);
  wait_clk_tick(1);
  i_tx_valid <= 1'b0;
  @(negedge o_tx_busy);
  wait_clk_tick(1);
end
endtask


endmodule